package ink.wgink.gateway.dao.route.impl;

import ink.wgink.gateway.consts.ISystemConst;
import ink.wgink.gateway.dao.BaseDao;
import ink.wgink.gateway.dao.route.IRouteDao;
import ink.wgink.gateway.pojo.dtos.route.RouteDTO;
import ink.wgink.gateway.pojo.pos.route.RoutePO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationOperation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: RouteDaoImpl
 * @Description: 路由
 * @Author: wanggeng
 * @Date: 2021/4/13 5:32 下午
 * @Version: 1.0
 */
@Slf4j
@Repository
public class RouteDaoImpl extends BaseDao implements IRouteDao {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public void save(RoutePO routePO) {
        setSave(routePO);
        mongoTemplate.save(routePO);
    }

    @Override
    public void remote(List<String> ids) {
        if (ids.isEmpty()) {
            return;
        }
        Criteria queryCriteria = Criteria.where(ISystemConst.UUID);
        for (int i = 0; i < ids.size(); i++) {
            if (i > 0) {
                queryCriteria.and(ISystemConst.UUID);
            }
            queryCriteria.is(ids.get(i));
        }

        mongoTemplate.remove(new Query(queryCriteria), RoutePO.class);
    }

    @Override
    public void update(String uuid, RoutePO routePO) {
        Update update = new Update();
        update.set("system", routePO.getSystem());
        update.set("summary", routePO.getSummary());
        update.set("path", routePO.getPath());
        update.set("uri", routePO.getUri());
        setUpdate(update);
        mongoTemplate.updateMulti(new Query(Criteria.where(ISystemConst.UUID).is(uuid)), update, RoutePO.class);
    }

    @Override
    public RouteDTO get(String uuid) {
        return mongoTemplate.findOne(new Query(Criteria.where(ISystemConst.UUID).is(uuid)), RouteDTO.class);
    }

    @Override
    public List<RouteDTO> list(int page, int size) {
        Query query = new Query();
        query.limit(getLimit(size));
        query.skip(getSkip(page, size));
        return mongoTemplate.find(query, RouteDTO.class);
    }

    @Override
    public Long count() {
        List<AggregationOperation> operations = new ArrayList<>();
        operations.add(Aggregation.group(ISystemConst.UUID).count().as(ISystemConst.COUNT));
        Aggregation aggregation = Aggregation.newAggregation(operations);
        AggregationResults<Map> results = mongoTemplate.aggregate(aggregation, COLLECTION_NAME, Map.class);
        return Long.parseLong(results.getMappedResults().get(0).get(ISystemConst.COUNT).toString());
    }
}
